Attribute VB_Name = "umFish20Ex"
' ------------------------------------------------------------------------------
'
' umFish20Ex(umFish20Ex.bas)                              Ulrich Mller 13.06.01
'                                                         Procida 400NT    VB6.0
' Funktionen zum Betrieb der fischertechnik Interfaces
' auf Basis von umFish20.DLL v2.0 :
' (auch unter VB5 lauffhig, evtl. Meldungen beim Laden ignorieren)
'
' Laufzeitdaten       : ftiDCB     : record, kein Aufrufparameter
'                       ftiNotHalt : Abbruchwunsch
'                       nur ein Interface pro Programm ( & Slave)
' Unterbrechbarkeit   : Integration von DoEvents.
' Abbrechbarkeit      : Variable ftiNotHalt, die Escape-Taste und Timelimit
'                       (bei langlaufenden Funktionen)
' Refresh             : durch Pollen in umFish20.DLL gewhrleistet
' Kontrolle Interface : ftiOpenInterface / ftiCloseInterface
'                       (WinNT|parallel : ftiStartDriver / ftiStopDriver)
' Lesen Eingabe       : ftiGetInputs / ftiGetInput, ftiGetAnalog
' Schreiben Ausgabe   : ftiSetMotors / ftiSetMotor / ftiClearMotors,
'                       ftiSetLamp
' Warten auf Ereignis : ftiWaitForTime, ftiWaitForInput,
'                       ftiWaitForChange, ftiWaitForLow
' Abbruchwunsch       : ftiFinish

' ------------------------------------------------------------------------------

Option Explicit

Public Declare Function umOpenInterface Lib "umFish20.dll" _
  (ft As ftDCB, ByVal ComName$) As Long
Public Declare Function umCloseInterface Lib "umFish20.dll" _
  (ft As ftDCB) As Long
Public Declare Function umGetVersion Lib "umFish20.dll" () As Long
Public Declare Function umGetInput Lib "umFish20.dll" _
  (ft As ftDCB, ByVal InputNr&) As Boolean
Public Declare Function umSetMotor Lib "umFish20.dll" _
  (ft As ftDCB, ByVal MotorNr&, ByVal Richtung&) As Long
Public Declare Function umSetLamp Lib "umFish20.dll" _
  (ft As ftDCB, ByVal LampNr&, ByVal OnOff&) As Long

Public Declare Function umStartDriver Lib "umFish20.dll" () As Long
Public Declare Function umStopDriver Lib "umFish20.dll" () As Long

Public Declare Function GetAsyncKeyState Lib "user32" (ByVal vKey As Long) As Integer
Public Const VK_ESCAPE = &H1B
Public Declare Sub Sleep Lib "kernel32" (ByVal dwMsec&)
Public Declare Function GetTickCount Lib "kernel32" () As Long


Public Type ftDCB
  comI As Long                   ' --- PortHandle / OpenFlag (=0)
  PortID As Long                 '     IO-Adresse bei LPT
  LPTDelay As Long               '     LPT Verzgerungsfaktor
  LPTAnalog As Long              '     LPT Analogfaktor
  ECount As Long                 '     Anzahl E-Eingnge
  
  FID As Long                    ' --- MultiMediaTimer Handle
  PollInterval As Long           '     PollInterval des MM-Timers
  AnalogScan As Long             '     Pollen auch der Analogeingnge (0/1)
  
  OutputStatus As Long           ' --- Status der M-Ausgnge
  InputStatus As Long            '     Status der E-Eingnge
  Analogs(0 To 1) As Long        '     Werte der A-Eingnge
  Counters(1 To 16) As Long      '     Impulszhler an den E-Eingngen
End Type

Public Const ftiFehler = &H1FFFF
Public Const ftiAus = 0
Public Const ftiEin = 1
Public Const ftiLinks = 1
Public Const ftiRechts = 2

Public ftiDCB As ftDCB, ftiNotHalt As Boolean

Private EMaske(1 To 16) As Long


Public Function ftiOpenInterface(PortName$) As Long

' --- Verbindung zum Interface herstellen ----------

' --- EMaske initialisieren ---

  EMaske(1) = &H1
  EMaske(2) = &H2
  EMaske(3) = &H4
  EMaske(4) = &H8
  EMaske(5) = &H10
  EMaske(6) = &H20
  EMaske(7) = &H40
  EMaske(8) = &H80
  EMaske(9) = &H100
  EMaske(10) = &H200
  EMaske(11) = &H400
  EMaske(12) = &H800
  EMaske(13) = &H1000
  EMaske(14) = &H2000
  EMaske(15) = &H4000
  EMaske(16) = &H8000
  
  ftiNotHalt = False
  ftiOpenInterface = umOpenInterface(ftiDCB, PortName)
  
End Function

Public Function ftiCloseInterface() As Long

' --- Verbindung zum Interface beenden ----

  ftiCloseInterface = umCloseInterface(ftiDCB)
  
End Function


Public Function ftiGetInputs() As Long
  
' --- Auslesen Status aller Digitaleingnge ----

  ftiGetInputs = ftiDCB.InputStatus
  DoEvents
  
End Function

Public Function ftiGetInput(ByVal InputNr As Long) As Boolean
  
' --- Auslesen Status Digitaleingang InputNr ----------------

  ftiGetInput = umGetInput(ftiDCB, InputNr)
  DoEvents
  
End Function

Public Function ftiGetAnalog(ByVal AnalogNr As Long) As Long
  
' --- Auslesen Wert Analogeingang AnalogNr -----------------

  ftiGetAnalog = ftiDCB.Analogs(AnalogNr)
  DoEvents
  
End Function

Public Function ftiSetMotors(MotorStatus&) As Long

' --- Setzen aller Ausgnge ----------------------

  ftiDCB.OutputStatus = MotorStatus
  ftiSetMotors = ftiDCB.InputStatus
  
End Function

Public Function ftiSetMotor(MotorNr&, Direction&) As Long

' --- Setzen Ausgang MotorNr auf Direction --------------

  ftiSetMotor = umSetMotor(ftiDCB, MotorNr, Direction)
  
End Function

Public Function ftiClearMotors() As Long

' --- Ausschalten aller Ausgnge -------

  ftiDCB.OutputStatus = 0
  ftiClearMotors = ftiDCB.InputStatus
  
End Function

Public Function ftiSetLamp(LampNr&, OnOff&) As Long

' ---- Setzen eines "halben" Ausganges LampNr auf OnOff ----

  ftiSetLamp = umSetLamp(ftiDCB, LampNr, OnOff)
  
End Function

Public Function ftiWaitForInput(InputNr&, Optional OnOff As Boolean = True) As Long
  
' --- Warten bis Eingang = OnOff wird -------------------------------------
'     Abbruch bei ftiNotHalt, Escape

  Do
    DoEvents
    If ftiNotHalt Or (GetAsyncKeyState(VK_ESCAPE) <> 0) Then
      ftiWaitForInput = ftiFehler
      Exit Function
    End If
  Loop Until umGetInput(ftiDCB, InputNr) = OnOff
  ftiWaitForInput = ftiDCB.InputStatus
  
End Function

Public Function ftiWaitForChange(InputNr&, NrOfChanges&, Optional TermInputNr& = 0) As Long

' --- Warten auf eine oder mehrere (NrOfChanges) nderungen des Status
'     des Einganges InputNr bzw. TermInputNr = true ------------------
'     Abbruch bei ftiNotHalt, escape, AbbruchZeit, Fehler, Endtaster

Const AbbruchDelta = 800
Dim i&, res&, M2&, M3&, res1 As Boolean, res2 As Boolean, AbbruchZeit&

  ftiWaitForChange = ftiFehler
  res = umGetInput(ftiDCB, InputNr)
  res1 = (res = True)
  M2 = EMaske(InputNr)
  If TermInputNr = 0 Then
    M3 = 0
  Else
    M3 = EMaske(TermInputNr)
    res = ftiDCB.InputStatus
    If (res And M3) > 0 Then
      ftiWaitForChange = res
      Exit Function
    End If
  End If
  
  For i = 1 To NrOfChanges
    AbbruchZeit = GetTickCount + AbbruchDelta
    Do
      DoEvents
      res = ftiDCB.InputStatus
      If (res = ftiFehler) Or ftiNotHalt Or (GetAsyncKeyState(VK_ESCAPE) <> 0) _
                           Or ((res And M3) > 0) _
                           Or (GetTickCount > AbbruchZeit) Then Exit Function
      res2 = (res And M2) > 0
      Loop Until res1 <> res2
      res1 = res2
  Next i
  ftiWaitForChange = res
  
End Function

Public Function ftiWaitForLow(InputNr&) As Long

' --- Warten auf einen true/false Durchgang von InputNr ---
'     Abbruch bei ftiNotHalt, Escape, Fehler

Dim res&, M2&, res1 As Boolean, res2 As Boolean

  ftiWaitForLow = ftiFehler
  res = umGetInput(ftiDCB, InputNr)
  res1 = (res = True)
  M2 = EMaske(InputNr)
  
  If Not res1 Then
    Do
      DoEvents
      res = ftiDCB.InputStatus
      If (res = ftiFehler) Or ftiNotHalt _
          Or (GetAsyncKeyState(VK_ESCAPE) <> 0) Then Exit Function
      res2 = (res And M2) > 0
    Loop Until res2
  End If
  
  Do
    DoEvents
    res = ftiDCB.InputStatus
    If (res = ftiFehler) Or ftiNotHalt _
       Or (GetAsyncKeyState(VK_ESCAPE) <> 0) Then Exit Function
    res2 = (res And M2) > 0
  Loop Until Not res2
  ftiWaitForLow = res
  
End Function

Public Sub ftiWaitForTime(ByVal mSek&)

' --- eine vorgegebene Zeit warten -----
'     Abbruch bei ftiNotHalt, Escape ---

Dim sZeit&, nSek&, res&

  nSek = mSek
  sZeit = GetTickCount()
  Do
    DoEvents
  Loop Until ftiNotHalt Or ((GetTickCount() - sZeit) > nSek) _
       Or (GetAsyncKeyState(VK_ESCAPE) <> 0) Or res = ftiFehler
       
End Sub

Public Function ftiFinish(InputNr&) As Boolean

' --- Feststellen eines Endewunsches : ftiNotHalt, Escape, InputNr ----

  ftiFinish = False
  If ftiNotHalt Or (GetAsyncKeyState(VK_ESCAPE) <> 0) Then ftiFinish = True
  If InputNr > 0 Then If umGetInput(ftiDCB, InputNr) Then ftiFinish = True
  DoEvents
  
End Function
